/**
 * @(#)tsid.cpp 2002/02/25
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID2.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _TSID2
#define _TSID2

#include "tsid2.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <sys/stat.h>

#ifdef HAVE_UNIX
#  include <unistd.h>
#  include <sys/fcntl.h>
#  include <dirent.h>
#else
#  include <direct.h> // mkdir
#  include "win32-dirent.h"
#  define  snprintf _snprintf
#  define  PATH_SEP '\\'
#endif

/**
 * Construct the TSID2 with passed base directory
 *
 * @param engine the type of the engine
 * @param number_t the number of time item to store in the list
 * @param number_n the number of number item  to store in the list
 * @param absBasePath the base path where there are the sidt2 files
 */
TSID2::TSID2(TBLIST::engineType engine, unsigned int number_t,
             unsigned int number_n, const char *absBasePath) {
  WDEBUG2("Construct the TSID2\n");

  DIR *handle;              // handle for file
  int err;                  // error is not 0 in file operation

  versionString="tsid2 1.00 by Ice Team Free Software Group (C) 2000-2002";

  tsidStatus=true;
  //lastError=NO_TSID_ERROR;

  pSidt=NULL;
  pList=NULL;
  TSID2::absBasePath=NULL;
  homeDir=NULL;
  bufferMD5=NULL;

  // look for the base path
  if (absBasePath==NULL) {                          // NULL -> use standard
    if (!getHomeDir()) {
      WDEBUG2("Error: not home directory found: exit\n");
      return;
    } else {
        absBasePath=homeDir;
      }

    TSID2::absBasePath=new char[strlen(absBasePath)+strlen(TSID2DIR)+2];
    if (TSID2::absBasePath==NULL) {
      // no memory error
      tsidStatus=false;
      return;
    }
    sprintf(TSID2::absBasePath, "%s/%s", absBasePath, TSID2DIR);
  } else {
      TSID2::absBasePath=new char[strlen(absBasePath)+1];
      if (TSID2::absBasePath==NULL) {
        // no memory error
        tsidStatus=false;
        return;
      }
      strcpy(TSID2::absBasePath, absBasePath);
    }

  convertDirSeperator(TSID2::absBasePath);

  // verify if base directory tree is present
  if ((handle=opendir(TSID2::absBasePath))==NULL) {
                                              // not present, must create it
#ifdef HAVE_UNIX
    err=mkdir(TSID2::absBasePath, O_CREAT);
    err+=chmod(TSID2::absBasePath, S_IRUSR | S_IWUSR | S_IEXEC);
#else
    err=mkdir(TSID2::absBasePath);
#endif

    WDEBUG2("Creating directory :\""<<TSID2::absBasePath<<"\"\n");
    if (err) {                                // error creating directory ?
      //lastError=DIR_CREATE;                   // error creating directory
      tsidStatus=false;
      return;
    }
  } else {                                    // present, close handle
      closedir(handle);
    }

  // construct tblist
  pList=new TBLIST(engine, TSID2::absBasePath, number_t, number_n, this);
  //if (!pList) {
  //  //error
  //}
}

/**
 * Destruct the TSID
 */
TSID2::~TSID2() {
  WDEBUG2("Destruct the TSID2\n");

  flush();

  if (pList!=NULL) delete pList;

  if (homeDir!=NULL) delete[] homeDir;
  if (absBasePath!=NULL) delete[] absBasePath;
  if (bufferMD5!=NULL) delete[] bufferMD5;
}

/**
 * Add the listen time to the corrisponding MD5 file
 * Use MD5 as tune name, absFilePath as absolute path of the last know
 * position of the tune; length<>0 if it is known.
 *
 * @param absSidFileName last known position of absolute sid file path and name
 * @param time the number of listened seconds
 * @param tune the subtune number
 * @param length the length of the tune or 0
 * @return true if the operation is ok
 */
bool TSID2::addTime (const char *MD5, const char *absSidFileName, unsigned int time,
                     unsigned int tune, unsigned int length) {

  WDEBUG2("Add "<<time<<" of #"<<tune<<" "<<MD5<<" with length "<< length<<endl);

  //lastError=NO_TSID_ERROR;

  // skip if low listen time
  if (time < MIN_LISTEN_TIME) {
    WDEBUG2("Skip operation due to low time\n");
    return true;
  }

/**** temp disable
  //make a copy of passed MD5
  char *cMD5=new char[strlen(MD5)+1];
  if (cMD5==NULL) {
    //lastError=NO_MEMORY;
    return false;
  }
  strcpy(cMD5, MD5);
*****/

  //update the tblist
  pList->setLast(absSidFileName);      // remember last name
////  pList->add(cMD5, tune, time, 1);

  pList->add(MD5, tune, time, 1);

/****** temp disable until fixed tblist
  if (bufferMD5!=NULL) {
    if (strcmp(cMD5, bufferMD5)==0) {
      // add the information to current song
      if (pSidt==NULL) {                     // this may be null if flush is called by tblist
        pSidt=new SIDT2(cMD5, absSidFileName, absBasePath);
      }
      pSidt->add(tune, time, 1, length);

      delete[] bufferMD5;                    // Remove one of the duplicate buffers
      bufferMD5=cMD5;                         // remember this new song name
    } else {

        if (!flush()) {                      // flush information
          return false;
        }
                                             // use a new sidt for this song
        pSidt=new SIDT2(cMD5, absSidFileName, absBasePath);
        pSidt->add(tune, time, 1, length);   // add information
        delete[] bufferMD5;
        bufferMD5=cMD5;                      // remember this new song name
      }
  } else {                                   // use a new sidt for this song
      pSidt=new SIDT2(cMD5, absSidFileName, absBasePath);
      pSidt->add(tune, time, 1, length);     // add information
      bufferMD5=cMD5;                        // remember this new song name
    }
****/

/*** temp fix for let tblist counting always rigth ***/
  if (pSidt==NULL) {                     // this may be null if flush is called by tblist
    pSidt=new SIDT2(MD5, absSidFileName, absBasePath);
  }
  pSidt->add(tune, time, 1, length);
  if (!flush()) {                      // flush information
    return false;
  }
/*** ***/

  return true;
}

/**
 * Add the listen time using a struct record of information
 *
 * @param tsidRecord the record of information
 * @return true if the operation is ok
 */
bool TSID2::addTime (struct tsidRecord rec) {
  return addTime(rec.MD5, rec.absSidFileName, rec.time, rec.tune, rec.length);
}

/**
 * Flush the information not saved to file
 *
 * @return true if operation is ok
 */
bool TSID2::flush() {
  WDEBUG2("Flush the information to file\n");

  //lastError=NO_TSID_ERROR;

  if (pSidt!=NULL) {
    if (!pSidt->read()) {                    // add just present information
      //lastError=READ_FILE;
      return false;
    }
    if (!pSidt->write()) {                   // write collected information
      //lastError=WRITE_FILE;
      return false;
    }
    delete pSidt;
    pSidt=NULL;
  }
  return true;
}

/**
 * Get home directory of user
 * The private variables homeDir is chaged
 * Note: this code is give by Simon White
 *
 * @return true if operation is ok
 */
bool TSID2::getHomeDir() {
  char *path = (char *) getenv ("HOME");
  //lastError=NO_TSID_ERROR;

  if (!path)
      path = (char *) getenv ("windir");
  if (!path) {
      //lastError=NO_USER;  // no user information
      return tsidStatus=false;
  }

  homeDir=new char[strlen(path)+1];
  if (homeDir==NULL) {
      //lastError=NO_MEMORY;  // no memory available
      return tsidStatus=false;
  }
  strcpy(homeDir, path);
  return true;
}

/**
 * Convert platform specific directory seperator to ansi
 *
 * @param filename the filename to made with '/'
 */
void TSID2::convertDirSeperator(char *filename) {
#ifndef HAVE_UNIX
  char *p = filename;
  while (*p != '\0') {
    if (*p == PATH_SEP) *p = '/';
    p++;
  }
#endif
}

#endif // _TSID2.CPP


