/**
 * @(#)tsidexaminer2.cpp 2002/02/25
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID2.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _TSIDEXAMINER2
#define _TSIDEXAMINER2

#include "tsidexaminer2.h"
#include "sidt2.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <sys/stat.h>

#ifdef HAVE_UNIX
#  include <unistd.h>
#  include <sys/fcntl.h>
#  include <dirent.h>
#elif __BORLANDC__
#  include <dirent.h>
#else
#  include "win32-dirent.h"
#  define S_ISDIR(st_mode) (st_mode & S_IFDIR)
#  define S_ISREG(st_mode) (st_mode & S_IFREG)
#endif

#define MAX_PATH_LEN  128
#define NLENGTH(dirent) ((int)strlen((dirent)->d_name))
#define strequ(s1, s2) (strcmp((s1),(s2)) == 0)

/**
 * Construnct the TSIDExaminer
 *
 * @param absBasePath the base path where there are the sidt2 files
 */
TSIDExaminer2::TSIDExaminer2(const char *absBasePath) {
  SDEBUG2("Construct the TSIDExaminer2 with: " << absBasePath << endl);

  startList=NULL;
  endList=NULL;
  listPointer=NULL;

  // test for base present
  if (absBasePath == NULL) {
    //lastError=NO_BASE_ERROR;     // no base path for the stored information
    //sidtStatus=false;            // tsid2 cannot continue
    return;
  }

  // make a phisical copy of the passed argument
  TSIDExaminer2::absBasePath=new char[strlen(absBasePath)+1];

  if (TSIDExaminer2::absBasePath == NULL) {
    //lastError=NO_MEMORY;            // no more memory
    //sidtStatus=false;               // sidt cannot go away
    return;
  }
  strcpy(TSIDExaminer2::absBasePath, absBasePath);
}

/**
 * Destruct the TSIDExaminer2
 */
TSIDExaminer2::~TSIDExaminer2() {
  SDEBUG2("Destruct the TSIDExaminer2\n");

  if (absBasePath != NULL) delete[] absBasePath;
  
  if (startList != NULL) {
    tuneItem *pTmp;                          // temp pointer

    while (startList->next != NULL) {
      pTmp=startList;                        // remember the start

      delete[] pTmp->rec.MD5;
      delete[] pTmp->rec.path;
      delete[] pTmp->rec.name;
      
      startList=startList->next;             // the new list
      delete pTmp;                           // release memory
    }
  }
}

/**
 * Analize the tsid2 files for retriving the information
 *
 * @return true if the operation is ok
 */
bool TSIDExaminer2::analise() {
  dirent *dp;
  struct stat st;
  DIR    *dirp;
  char   nbuf[MAX_PATH_LEN];
  int    len;
  char   name[37];

  SDEBUG2("Scan directory " << absBasePath << endl);

  //lastError=NO_TSIDE_ERROR;                  // no error

  dirp=opendir(absBasePath);

  if (dirp==NULL) {
    SDEBUG2(absBasePath << " unreadable\n");
    //lastError=DIR_READ;                      // read error of directory
    return false;
  }

  while ((dp=readdir(dirp)) != NULL) {
    if (strequ(dp->d_name,".") || strequ(dp->d_name,"..")) {
      continue;
    }
    len=strlen(absBasePath);
    if (len+NLENGTH(dp)+1 < MAX_PATH_LEN-1) {
      strcpy(nbuf, absBasePath);
      if (len != 0) {
        nbuf[len++]='/';
      }
      strcpy(nbuf+len, dp->d_name);

      stat(nbuf, &st);

      if (S_ISREG(st.st_mode) && strlen(dp->d_name)==36 ) { //36=length Md5 + ext
        strcpy(name, dp->d_name);
        if (strcmp(&name[33], "dat")==0) {
          name[32]='\0';
          addFileToList(name);                 // add the file
        }
      }
    } else {
        SDEBUG2(absBasePath << "/" << dp->d_name << ": pathname too long\n");
        //lastError=TOO_LONG;
      }
  }
  closedir(dirp);

  return true;
}

/**
 * Insert the information of SIDT2 file into the list
 *
 * @param nfile name of file
 * @return true if the operation is ok
 */
bool TSIDExaminer2::addFileToList(char *nfile) {
  SDEBUG2("Add information from file " << nfile << endl);

  // pointer to sidt file manager
  SIDT2 *pSidt=new SIDT2(nfile, "#/#", absBasePath);

  tuneItem *pItem;               // a item for the list

  if (!*pSidt) {
    SDEBUG2("SIDT never initialize\n");
    //lastError=EXT_ERROR;
    //dinamicError=pSidt->getError();
    return false;
  }

  if (!pSidt->read()) {
    SDEBUG2("Error reading SIDT information\n");
    //dinamicError=pSidt->getError();
    //lastError=EXT_ERROR;
    return false;
  }

  pSidt->initForGet();                       // init for getting next subtune

  pItem=new tuneItem;

  // scan all the subtunes that are present
  while (pSidt->getNext(&pItem->rec.tune, &pItem->rec.second, 
                        &pItem->rec.number, &pItem->rec.length)) {

    // get pondered score
    //getScore(pItem, pSidName);

    SDEBUG2("Add song #" << pItem->rec.tune << endl);

    if (startList==NULL) {                   // list empty ?
      startList=pItem;                       // this is the first element
      endList=pItem;                         // and the last too
    } else {                                 // list not empty
        endList->next=pItem;                 // next point to the last
        endList=pItem;                       // this is the last
      }
    // pItem must be completed with the path and file name
    pItem->rec.path=pSidt->getReadSidPath();
    pItem->rec.name=pSidt->getReadSidName();
    pItem->rec.MD5=pSidt->getMD5();

    pItem->next=NULL;                        // there's not other
    pItem=new tuneItem;
  }

  delete pItem;
  delete pSidt;

  return true;
}

/**
 * Sort the list for the passed type
 *
 * @param type the type of sort to apply
 */
void TSIDExaminer2::sort(sortType type) {
  tuneItem *actual;        // actual pointer
  tuneItem *start;         // actual start pointer (upper, all are ordered)
  tuneItem tmp;            // tmp for swap

  SDEBUG2("Sort List\n");

  // swap element macro
  #define swap(a,b) \
     tmp.rec.path=a->rec.path; \
     tmp.rec.name=a->rec.name; \
     tmp.rec.MD5=a->rec.MD5; \
     tmp.rec.tune=a->rec.tune; \
     tmp.rec.second=a->rec.second; \
     tmp.rec.number=a->rec.number; \
     tmp.rec.length=a->rec.length; \
     a->rec.path=b->rec.path; \
     a->rec.name=b->rec.name; \
     a->rec.MD5=b->rec.MD5; \
     a->rec.tune=b->rec.tune; \
     a->rec.second=b->rec.second; \
     a->rec.number=b->rec.number; \
     a->rec.length=b->rec.length; \
     b->rec.path=tmp.rec.path; \
     b->rec.name=tmp.rec.name; \
     b->rec.MD5=tmp.rec.MD5; \
     b->rec.tune=tmp.rec.tune; \
     b->rec.second=tmp.rec.second; \
     b->rec.number=tmp.rec.number; \
     b->rec.length=tmp.rec.length;

  // use a bubble sort algorithm (oh, what I see)
  if (startList==NULL) return;               // list has no element
  if (startList->next==NULL) return;         // list has one element

  for (start=startList; start->next!=NULL; start=start->next)
    for (actual=start->next; actual!=NULL; actual=actual->next) {
      switch (type) {
        case TIME:
          if (actual->rec.second > start->rec.second) {
             swap(actual, start);
          }
          break;
        case NUMBER:
          if (actual->rec.number > start->rec.number) {
             swap(actual, start);
          }
          break;
       // case SORT_FOR_SCORE:
       //   if (actual->score > start->score) {
       //     swap(actual, start);
       //   }
        // default: do nothing
      }
    }
}

/**
 * Init the engin for getting the next song information from list
 */
void TSIDExaminer2::initForGet() {
  SDEBUG2("Init engine for getting songs\n");
  
  listPointer=startList;   // start of element in list
}

/**
 * Get next tune information from the list
 * The returned strings are a just created copy
 *
 * @param rec a record of information
 * @return true if the record is present
 */
bool TSIDExaminer2::getNext(struct TRec *rec) {
  SDEBUG2("Get next is called" << endl);
  // finish element or no list available?

  if (listPointer==NULL) {
    return false;
  }

  size_t size=strlen(listPointer->rec.MD5);
  size_t size1=strlen(listPointer->rec.path);
  size_t size2=strlen(listPointer->rec.name);
  rec->MD5=new char[size+1];
  rec->path=new char[size1+1];
  rec->name=new char[size2+1];

  if ((rec->MD5==NULL) || (rec->name==NULL) || (rec->path==NULL)) {
      //lastError=NO_MEMORY;
      //tsidexaminerStatus=false;
      return false;         // this error stop list creation
  }

  // copy the strings
  strncpy(rec->MD5, listPointer->rec.MD5, size+1);
  strncpy(rec->path, listPointer->rec.path, size1+1);
  strncpy(rec->name, listPointer->rec.name, size2+1);

  SDEBUG2("Get next: " << rec->MD5 << "..." << endl);

  rec->tune=listPointer->rec.tune;
  rec->second=listPointer->rec.second;
  rec->number=listPointer->rec.number;
  rec->length=listPointer->rec.length;

  SDEBUG2("...#" << rec->tune << ", " << rec->second << ", " << rec->number << ", "
          << rec->length << endl);

  listPointer=listPointer->next;

  return true;
}

#endif // _TSIDExaminer2.CPP


