/**
 * @(#)tsidexaminer2.h 2002/02/25
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID2.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _TSIDEXAMINER2_H
#define _TSIDEXAMINER2_H

#include "version2.h"

// show debug string if DEBUG is defined
#ifdef  DEBUG_TSIDEXAMINER2
#  define SDEBUG2(a) \
    cerr << "[TSIDEXAMINER2] " << a;
#else
#  define SDEBUG2(a) ;
#endif

/**
 * TSIDEXAMINER2 class
 *
 * This class examines the TSID2 tree.
 * It collects all the data and then processes for giving useful information
 *
 * @author ice
 * @version 1.00 25/02/2002
 */
class TSIDExaminer2 {
  public:

    // enum for the sorted type to apply
    enum sortType {
      TIME,
      NUMBER,
      SCORE
    };
    
    // a TSIDExaminer record of information
    struct TRec {
      char *MD5;       // MD5 of the tune
      char *path;      // last known path of the tune
      char *name;      // last known name of the tune
      unsigned int tune;             // subtune number
      unsigned long second;          // listen second
      unsigned long number;          // number of listen time
      unsigned long length;          // length of the tune if known or 0
    };
    
    // contains information of 1 tune in TSID tree
    struct tuneItem {
      struct TRec rec;               // record of information
      tuneItem *next;                // next element of list
    };

    /**
     * Construnct the TSIDExaminer
     *
     * @param absBasePath the base path where there are the sidt2 files
     */
    TSIDExaminer2(const char *absBasePath);

    /**
     * Destruct the TSIDExaminer2
     */
    ~TSIDExaminer2();

    /**
     * Return the state of TSIDExaminer2: false means that TSIDExaminer2 cannot
     * continue
     */
    operator bool() {
      return tsidexaminerStatus;
    }

    /**
     * Analize the tsid2 files for retriving the information
     *
     * @return true if the operation is ok
     */
    bool analise();

    /**
     * Insert the information of SIDT2 file into the list
     *
     * @param nfile name of file
     * @return true if the operation is ok
     */
    bool addFileToList(char *nfile);


    /**
     * Sort the list for the passed type
     *
     * @param type the type of sort to apply
     */
    void sort(sortType type);

    /**
     * Init the engin for getting the next song information from list
     */
    void initForGet();

    /**
     * Get next tune information from the list
     * The returned strings are a just created copy
     *
     * @param rec a record of information
     * @return true if the record is present
     */
    bool getNext(struct TRec *rec);
    
  protected:
    // pointer to start of list of tune
    tuneItem *startList;

    // pointer to last element in the list
    tuneItem *endList;

  private:
    // Actual pointer in list being scanned
    tuneItem *listPointer;

    // true if tsidexaminer2 can continue to work correctly
    bool tsidexaminerStatus;

    // The absolute base path where there are the sidt2 files
    char *absBasePath;
};

#endif // _SIDTEXAMINER2_H
